@@ -4,13 +4,10 @@ import pprint
 
 import _pytest._code
 import py
-try:
-    from collections import Sequence
-except ImportError:
-    Sequence = list
+import six
+from ..compat import Sequence
 
-
-u = py.builtin._totext
+u = six.text_type
 
 # The _reprcompare attribute on the util module is used by the new assertion
 # interpretation code and assertion rewriter to detect this plugin was
@@ -23,7 +20,7 @@ _reprcompare = None
 # with non-ascii characters (see issue 877 and 1379)
 def ecu(s):
     try:
-        return u(s, 'utf-8', 'replace')
+        return u(s, "utf-8", "replace")
     except TypeError:
         return s
 
@@ -41,7 +38,7 @@ def format_explanation(explanation):
     explanation = ecu(explanation)
     lines = _split_explanation(explanation)
     result = _format_lines(lines)
-    return u('\n').join(result)
+    return u("\n").join(result)
 
 
 def _split_explanation(explanation):
@@ -51,13 +48,13 @@ def _split_explanation(explanation):
     Any other newlines will be escaped and appear in the line as the
     literal '\n' characters.
     """
-    raw_lines = (explanation or u('')).split('\n')
+    raw_lines = (explanation or u("")).split("\n")
     lines = [raw_lines[0]]
-    for l in raw_lines[1:]:
-        if l and l[0] in ['{', '}', '~', '>']:
-            lines.append(l)
+    for values in raw_lines[1:]:
+        if values and values[0] in ["{", "}", "~", ">"]:
+            lines.append(values)
         else:
-            lines[-1] += '\\n' + l
+            lines[-1] += "\\n" + values
     return lines
 
 
@@ -74,24 +71,24 @@ def _format_lines(lines):
     stack = [0]
     stackcnt = [0]
     for line in lines[1:]:
-        if line.startswith('{'):
+        if line.startswith("{"):
             if stackcnt[-1]:
-                s = u('and   ')
+                s = u("and   ")
             else:
-                s = u('where ')
+                s = u("where ")
             stack.append(len(result))
             stackcnt[-1] += 1
             stackcnt.append(0)
-            result.append(u(' +') + u('  ') * (len(stack) - 1) + s + line[1:])
-        elif line.startswith('}'):
+            result.append(u(" +") + u("  ") * (len(stack) - 1) + s + line[1:])
+        elif line.startswith("}"):
             stack.pop()
             stackcnt.pop()
             result[stack[-1]] += line[1:]
         else:
-            assert line[0] in ['~', '>']
+            assert line[0] in ["~", ">"]
             stack[-1] += 1
-            indent = len(stack) if line.startswith('~') else len(stack) - 1
-            result.append(u('  ') * indent + line[1:])
+            indent = len(stack) if line.startswith("~") else len(stack) - 1
+            result.append(u("  ") * indent + line[1:])
     assert len(stack) == 1
     return result
 
@@ -109,10 +106,10 @@ def assertrepr_compare(config, op, left, right):
     left_repr = py.io.saferepr(left, maxsize=int(width // 2))
     right_repr = py.io.saferepr(right, maxsize=width - len(left_repr))
 
-    summary = u('%s %s %s') % (ecu(left_repr), op, ecu(right_repr))
+    summary = u("%s %s %s") % (ecu(left_repr), op, ecu(right_repr))
 
     def issequence(x):
-        return (isinstance(x, (list, tuple, Sequence)) and not isinstance(x, basestring))
+        return isinstance(x, Sequence) and not isinstance(x, basestring)
 
     def istext(x):
         return isinstance(x, basestring)
@@ -130,10 +127,10 @@ def assertrepr_compare(config, op, left, right):
         except TypeError:
             return False
 
-    verbose = config.getoption('verbose')
+    verbose = config.getoption("verbose")
     explanation = None
     try:
-        if op == '==':
+        if op == "==":
             if istext(left) and istext(right):
                 explanation = _diff_text(left, right, verbose)
             else:
@@ -149,14 +146,17 @@ def assertrepr_compare(config, op, left, right):
                         explanation.extend(expl)
                     else:
                         explanation = expl
-        elif op == 'not in':
+        elif op == "not in":
             if istext(left) and istext(right):
                 explanation = _notin_text(left, right, verbose)
     except Exception:
         explanation = [
-            u('(pytest_assertion plugin: representation of details failed.  '
-              'Probably an object has a faulty __repr__.)'),
-            u(_pytest._code.ExceptionInfo())]
+            u(
+                "(pytest_assertion plugin: representation of details failed.  "
+                "Probably an object has a faulty __repr__.)"
+            ),
+            u(_pytest._code.ExceptionInfo()),
+        ]
 
     if not explanation:
         return None
@@ -173,20 +173,35 @@ def _diff_text(left, right, verbose=False):
     If the input are bytes they will be safely converted to text.
     """
     from difflib import ndiff
+
     explanation = []
-    if isinstance(left, py.builtin.bytes):
-        left = u(repr(left)[1:-1]).replace(r'\n', '\n')
-    if isinstance(right, py.builtin.bytes):
-        right = u(repr(right)[1:-1]).replace(r'\n', '\n')
+
+    def escape_for_readable_diff(binary_text):
+        """
+        Ensures that the internal string is always valid unicode, converting any bytes safely to valid unicode.
+        This is done using repr() which then needs post-processing to fix the encompassing quotes and un-escape
+        newlines and carriage returns (#429).
+        """
+        r = six.text_type(repr(binary_text)[1:-1])
+        r = r.replace(r"\n", "\n")
+        r = r.replace(r"\r", "\r")
+        return r
+
+    if isinstance(left, six.binary_type):
+        left = escape_for_readable_diff(left)
+    if isinstance(right, six.binary_type):
+        right = escape_for_readable_diff(right)
     if not verbose:
         i = 0  # just in case left or right has zero length
         for i in range(min(len(left), len(right))):
             if left[i] != right[i]:
                 break
         if i > 42:
-            i -= 10                 # Provide some context
-            explanation = [u('Skipping %s identical leading '
-                             'characters in diff, use -v to show') % i]
+            i -= 10  # Provide some context
+            explanation = [
+                u("Skipping %s identical leading " "characters in diff, use -v to show")
+                % i
+            ]
             left = left[i:]
             right = right[i:]
         if len(left) == len(right):
@@ -194,36 +209,48 @@ def _diff_text(left, right, verbose=False):
                 if left[-i] != right[-i]:
                     break
             if i > 42:
-                i -= 10     # Provide some context
-                explanation += [u('Skipping %s identical trailing '
-                                  'characters in diff, use -v to show') % i]
+                i -= 10  # Provide some context
+                explanation += [
+                    u(
+                        "Skipping %s identical trailing "
+                        "characters in diff, use -v to show"
+                    )
+                    % i
+                ]
                 left = left[:-i]
                 right = right[:-i]
     keepends = True
-    explanation += [line.strip('\n')
-                    for line in ndiff(left.splitlines(keepends),
-                                      right.splitlines(keepends))]
+    if left.isspace() or right.isspace():
+        left = repr(str(left))
+        right = repr(str(right))
+        explanation += [u"Strings contain only whitespace, escaping them using repr()"]
+    explanation += [
+        line.strip("\n")
+        for line in ndiff(left.splitlines(keepends), right.splitlines(keepends))
+    ]
     return explanation
 
 
 def _compare_eq_iterable(left, right, verbose=False):
     if not verbose:
-        return [u('Use -v to get the full diff')]
+        return [u("Use -v to get the full diff")]
     # dynamic import to speedup pytest
     import difflib
 
     try:
         left_formatting = pprint.pformat(left).splitlines()
         right_formatting = pprint.pformat(right).splitlines()
-        explanation = [u('Full diff:')]
+        explanation = [u("Full diff:")]
     except Exception:
         # hack: PrettyPrinter.pformat() in python 2 fails when formatting items that can't be sorted(), ie, calling
         # sorted() on a list would raise. See issue #718.
         # As a workaround, the full diff is generated by using the repr() string of each item of each container.
         left_formatting = sorted(repr(x) for x in left)
         right_formatting = sorted(repr(x) for x in right)
-        explanation = [u('Full diff (fallback to calling repr on each item):')]
-    explanation.extend(line.strip() for line in difflib.ndiff(left_formatting, right_formatting))
+        explanation = [u("Full diff (fallback to calling repr on each item):")]
+    explanation.extend(
+        line.strip() for line in difflib.ndiff(left_formatting, right_formatting)
+    )
     return explanation
 
 
@@ -231,16 +258,18 @@ def _compare_eq_sequence(left, right, verbose=False):
     explanation = []
     for i in range(min(len(left), len(right))):
         if left[i] != right[i]:
-            explanation += [u('At index %s diff: %r != %r')
-                            % (i, left[i], right[i])]
+            explanation += [u("At index %s diff: %r != %r") % (i, left[i], right[i])]
             break
     if len(left) > len(right):
-        explanation += [u('Left contains more items, first extra item: %s')
-                        % py.io.saferepr(left[len(right)],)]
+        explanation += [
+            u("Left contains more items, first extra item: %s")
+            % py.io.saferepr(left[len(right)])
+        ]
     elif len(left) < len(right):
         explanation += [
-            u('Right contains more items, first extra item: %s') %
-            py.io.saferepr(right[len(left)],)]
+            u("Right contains more items, first extra item: %s")
+            % py.io.saferepr(right[len(left)])
+        ]
     return explanation
 
 
@@ -249,11 +278,11 @@ def _compare_eq_set(left, right, verbose=False):
     diff_left = left - right
     diff_right = right - left
     if diff_left:
-        explanation.append(u('Extra items in the left set:'))
+        explanation.append(u("Extra items in the left set:"))
         for item in diff_left:
             explanation.append(py.io.saferepr(item))
     if diff_right:
-        explanation.append(u('Extra items in the right set:'))
+        explanation.append(u("Extra items in the right set:"))
         for item in diff_right:
             explanation.append(py.io.saferepr(item))
     return explanation
@@ -262,46 +291,48 @@ def _compare_eq_set(left, right, verbose=False):
 def _compare_eq_dict(left, right, verbose=False):
     explanation = []
     common = set(left).intersection(set(right))
-    same = dict((k, left[k]) for k in common if left[k] == right[k])
+    same = {k: left[k] for k in common if left[k] == right[k]}
     if same and verbose < 2:
-        explanation += [u('Omitting %s identical items, use -vv to show') %
-                        len(same)]
+        explanation += [u("Omitting %s identical items, use -vv to show") % len(same)]
     elif same:
-        explanation += [u('Common items:')]
+        explanation += [u("Common items:")]
         explanation += pprint.pformat(same).splitlines()
-    diff = set(k for k in common if left[k] != right[k])
+    diff = {k for k in common if left[k] != right[k]}
     if diff:
-        explanation += [u('Differing items:')]
+        explanation += [u("Differing items:")]
         for k in diff:
-            explanation += [py.io.saferepr({k: left[k]}) + ' != ' +
-                            py.io.saferepr({k: right[k]})]
+            explanation += [
+                py.io.saferepr({k: left[k]}) + " != " + py.io.saferepr({k: right[k]})
+            ]
     extra_left = set(left) - set(right)
     if extra_left:
-        explanation.append(u('Left contains more items:'))
-        explanation.extend(pprint.pformat(
-            dict((k, left[k]) for k in extra_left)).splitlines())
+        explanation.append(u("Left contains more items:"))
+        explanation.extend(
+            pprint.pformat({k: left[k] for k in extra_left}).splitlines()
+        )
     extra_right = set(right) - set(left)
     if extra_right:
-        explanation.append(u('Right contains more items:'))
-        explanation.extend(pprint.pformat(
-            dict((k, right[k]) for k in extra_right)).splitlines())
+        explanation.append(u("Right contains more items:"))
+        explanation.extend(
+            pprint.pformat({k: right[k] for k in extra_right}).splitlines()
+        )
     return explanation
 
 
 def _notin_text(term, text, verbose=False):
     index = text.find(term)
     head = text[:index]
-    tail = text[index + len(term):]
+    tail = text[index + len(term) :]
     correct_text = head + tail
     diff = _diff_text(correct_text, text, verbose)
-    newdiff = [u('%s is contained here:') % py.io.saferepr(term, maxsize=42)]
+    newdiff = [u("%s is contained here:") % py.io.saferepr(term, maxsize=42)]
     for line in diff:
-        if line.startswith(u('Skipping')):
+        if line.startswith(u("Skipping")):
             continue
-        if line.startswith(u('- ')):
+        if line.startswith(u("- ")):
             continue
-        if line.startswith(u('+ ')):
-            newdiff.append(u('  ') + line[2:])
+        if line.startswith(u("+ ")):
+            newdiff.append(u("  ") + line[2:])
         else:
             newdiff.append(line)
     return newdiff
