@@ -1,13 +1,14 @@
 import inspect
 import warnings
 from collections import namedtuple
+from functools import reduce
 from operator import attrgetter
 
 import attr
 
 from ..deprecated import MARK_PARAMETERSET_UNPACKING, MARK_INFO_ATTRIBUTE
 from ..compat import NOTSET, getfslineno, MappingMixin
-from six.moves import map, reduce
+from six.moves import map
 
 
 EMPTY_PARAMETERSET_OPTION = "empty_parameter_set_mark"
@@ -24,9 +25,10 @@ def alias(name, warning=None):
 
 
 def istestfunc(func):
-    return hasattr(func, "__call__") and getattr(
-        func, "__name__", "<lambda>"
-    ) != "<lambda>"
+    return (
+        hasattr(func, "__call__")
+        and getattr(func, "__name__", "<lambda>") != "<lambda>"
+    )
 
 
 def get_empty_parameterset_mark(config, argnames, func):
@@ -39,18 +41,20 @@ def get_empty_parameterset_mark(config, argnames, func):
         raise LookupError(requested_mark)
     fs, lineno = getfslineno(func)
     reason = "got empty parameter set %r, function %s at %s:%d" % (
-        argnames, func.__name__, fs, lineno
+        argnames,
+        func.__name__,
+        fs,
+        lineno,
     )
     return mark(reason=reason)
 
 
 class ParameterSet(namedtuple("ParameterSet", "values, marks, id")):
-
     @classmethod
     def param(cls, *values, **kw):
         marks = kw.pop("marks", ())
         if isinstance(marks, MarkDecorator):
-            marks = marks,
+            marks = (marks,)
         else:
             assert isinstance(marks, (tuple, list, set))
 
@@ -87,7 +91,7 @@ class ParameterSet(namedtuple("ParameterSet", "values, marks, id")):
             argval = argval.args[-1]
         assert not isinstance(argval, ParameterSet)
         if legacy_force_tuple:
-            argval = argval,
+            argval = (argval,)
 
         if newmarks:
             warnings.warn(MARK_PARAMETERSET_UNPACKING)
@@ -107,7 +111,19 @@ class ParameterSet(namedtuple("ParameterSet", "values, marks, id")):
         ]
         del argvalues
 
-        if not parameters:
+        if parameters:
+            # check all parameter sets have the correct number of values
+            for param in parameters:
+                if len(param.values) != len(argnames):
+                    raise ValueError(
+                        'In "parametrize" the number of values ({}) must be '
+                        "equal to the number of names ({})".format(
+                            param.values, argnames
+                        )
+                    )
+        else:
+            # empty parameter set (likely computed at runtime): create a single
+            # parameter set with NOSET values, with the "empty parameter set" mark applied to it
             mark = get_empty_parameterset_mark(config, argnames, func)
             parameters.append(
                 ParameterSet(values=(NOTSET,) * len(argnames), marks=[mark], id=None)
@@ -120,9 +136,9 @@ class Mark(object):
     #: name of the mark
     name = attr.ib(type=str)
     #: positional arguments of the mark decorator
-    args = attr.ib(type="List[object]")
+    args = attr.ib()  # type: List[object]
     #: keyword arguments of the mark decorator
-    kwargs = attr.ib(type="Dict[str, object]")
+    kwargs = attr.ib()  # type: Dict[str, object]
 
     def combined_with(self, other):
         """
@@ -221,9 +237,18 @@ def get_unpacked_marks(obj):
     obtain the unpacked marks that are stored on an object
     """
     mark_list = getattr(obj, "pytestmark", [])
-
     if not isinstance(mark_list, list):
         mark_list = [mark_list]
+    return normalize_mark_list(mark_list)
+
+
+def normalize_mark_list(mark_list):
+    """
+    normalizes marker decorating helpers to mark objects
+
+    :type mark_list: List[Union[Mark, Markdecorator]]
+    :rtype: List[Mark]
+    """
     return [getattr(mark, "mark", mark) for mark in mark_list]  # unpack MarkDecorator
 
 
@@ -246,7 +271,7 @@ def store_legacy_markinfo(func, mark):
     if holder is None:
         holder = MarkInfo.for_mark(mark)
         setattr(func, mark.name, holder)
-    else:
+    elif isinstance(holder, MarkInfo):
         holder.add_mark(mark)
 
 
@@ -281,7 +306,18 @@ def _marked(func, mark):
 class MarkInfo(object):
     """ Marking object created by :class:`MarkDecorator` instances. """
 
-    _marks = attr.ib()
+    _marks = attr.ib(converter=list)
+
+    @_marks.validator
+    def validate_marks(self, attribute, value):
+        for item in value:
+            if not isinstance(item, Mark):
+                raise ValueError(
+                    "MarkInfo expects Mark instances, got {!r} ({!r})".format(
+                        item, type(item)
+                    )
+                )
+
     combined = attr.ib(
         repr=False,
         default=attr.Factory(
@@ -321,6 +357,7 @@ class MarkGenerator(object):
 
     will set a 'slowtest' :class:`MarkInfo` object
     on the ``test_function`` object. """
+
     _config = None
 
     def __getattr__(self, name):
@@ -350,7 +387,6 @@ MARK_GEN = MarkGenerator()
 
 
 class NodeKeywords(MappingMixin):
-
     def __init__(self, node):
         self.node = node
         self.parent = node.parent
@@ -397,6 +433,7 @@ class NodeMarkers(object):
         unstable api
 
     """
+
     own_markers = attr.ib(default=attr.Factory(list))
 
     def update(self, add_markers):
