@@ -9,7 +9,6 @@ import _pytest._code
 from _pytest.config import hookimpl
 from _pytest.outcomes import fail, skip, xfail
 from _pytest.python import transfer_markers, Class, Module, Function
-from _pytest.skipping import MarkEvaluator
 
 
 def pytest_pycollect_makeitem(collector, name, obj):
@@ -30,18 +29,19 @@ class UnitTestCase(Class):
 
     def setup(self):
         cls = self.obj
-        if getattr(cls, '__unittest_skip__', False):
+        if getattr(cls, "__unittest_skip__", False):
             return  # skipped
-        setup = getattr(cls, 'setUpClass', None)
+        setup = getattr(cls, "setUpClass", None)
         if setup is not None:
             setup()
-        teardown = getattr(cls, 'tearDownClass', None)
+        teardown = getattr(cls, "tearDownClass", None)
         if teardown is not None:
             self.addfinalizer(teardown)
         super(UnitTestCase, self).setup()
 
     def collect(self):
         from unittest import TestLoader
+
         cls = self.obj
         if not getattr(cls, "__test__", True):
             return
@@ -51,29 +51,30 @@ class UnitTestCase(Class):
         foundsomething = False
         for name in loader.getTestCaseNames(self.obj):
             x = getattr(self.obj, name)
-            if not getattr(x, '__test__', True):
+            if not getattr(x, "__test__", True):
                 continue
-            funcobj = getattr(x, 'im_func', x)
+            funcobj = getattr(x, "im_func", x)
             transfer_markers(funcobj, cls, module)
-            yield TestCaseFunction(name, parent=self)
+            yield TestCaseFunction(name, parent=self, callobj=funcobj)
             foundsomething = True
 
         if not foundsomething:
-            runtest = getattr(self.obj, 'runTest', None)
+            runtest = getattr(self.obj, "runTest", None)
             if runtest is not None:
                 ut = sys.modules.get("twisted.trial.unittest", None)
                 if ut is None or runtest != ut.TestCase.runTest:
-                    yield TestCaseFunction('runTest', parent=self)
+                    yield TestCaseFunction("runTest", parent=self)
 
 
 class TestCaseFunction(Function):
+    nofuncargs = True
     _excinfo = None
 
     def setup(self):
         self._testcase = self.parent.obj(self.name)
         self._fix_unittest_skip_decorator()
         self._obj = getattr(self._testcase, self.name)
-        if hasattr(self._testcase, 'setup_method'):
+        if hasattr(self._testcase, "setup_method"):
             self._testcase.setup_method(self._obj)
         if hasattr(self, "_request"):
             self._request._fillfixtures()
@@ -92,7 +93,7 @@ class TestCaseFunction(Function):
             setattr(self._testcase, "__name__", self.name)
 
     def teardown(self):
-        if hasattr(self._testcase, 'teardown_method'):
+        if hasattr(self._testcase, "teardown_method"):
             self._testcase.teardown_method(self._obj)
         # Allow garbage collection on TestCase instance attributes.
         self._testcase = None
@@ -103,26 +104,32 @@ class TestCaseFunction(Function):
 
     def _addexcinfo(self, rawexcinfo):
         # unwrap potential exception info (see twisted trial support below)
-        rawexcinfo = getattr(rawexcinfo, '_rawexcinfo', rawexcinfo)
+        rawexcinfo = getattr(rawexcinfo, "_rawexcinfo", rawexcinfo)
         try:
             excinfo = _pytest._code.ExceptionInfo(rawexcinfo)
         except TypeError:
             try:
                 try:
-                    l = traceback.format_exception(*rawexcinfo)
-                    l.insert(0, "NOTE: Incompatible Exception Representation, "
-                                "displaying natively:\n\n")
-                    fail("".join(l), pytrace=False)
+                    values = traceback.format_exception(*rawexcinfo)
+                    values.insert(
+                        0,
+                        "NOTE: Incompatible Exception Representation, "
+                        "displaying natively:\n\n",
+                    )
+                    fail("".join(values), pytrace=False)
                 except (fail.Exception, KeyboardInterrupt):
                     raise
-                except:
-                    fail("ERROR: Unknown Incompatible Exception "
-                         "representation:\n%r" % (rawexcinfo,), pytrace=False)
+                except:  # noqa
+                    fail(
+                        "ERROR: Unknown Incompatible Exception "
+                        "representation:\n%r" % (rawexcinfo,),
+                        pytrace=False,
+                    )
             except KeyboardInterrupt:
                 raise
             except fail.Exception:
                 excinfo = _pytest._code.ExceptionInfo()
-        self.__dict__.setdefault('_excinfo', []).append(excinfo)
+        self.__dict__.setdefault("_excinfo", []).append(excinfo)
 
     def addError(self, testcase, rawexcinfo):
         self._addexcinfo(rawexcinfo)
@@ -134,8 +141,7 @@ class TestCaseFunction(Function):
         try:
             skip(reason)
         except skip.Exception:
-            self._evalskip = MarkEvaluator(self, 'SkipTest')
-            self._evalskip.result = True
+            self._skipped_by_mark = True
             self._addexcinfo(sys.exc_info())
 
     def addExpectedFailure(self, testcase, rawexcinfo, reason=""):
@@ -157,11 +163,13 @@ class TestCaseFunction(Function):
         # implements the skipping machinery (see #2137)
         # analog to pythons Lib/unittest/case.py:run
         testMethod = getattr(self._testcase, self._testcase._testMethodName)
-        if (getattr(self._testcase.__class__, "__unittest_skip__", False) or
-                getattr(testMethod, "__unittest_skip__", False)):
+        if getattr(self._testcase.__class__, "__unittest_skip__", False) or getattr(
+            testMethod, "__unittest_skip__", False
+        ):
             # If the class or method was skipped.
-            skip_why = (getattr(self._testcase.__class__, '__unittest_skip_why__', '') or
-                        getattr(testMethod, '__unittest_skip_why__', ''))
+            skip_why = getattr(
+                self._testcase.__class__, "__unittest_skip_why__", ""
+            ) or getattr(testMethod, "__unittest_skip_why__", "")
             try:  # PY3, unittest2 on PY2
                 self._testcase._addSkip(self, self._testcase, skip_why)
             except TypeError:  # PY2
@@ -183,7 +191,8 @@ class TestCaseFunction(Function):
     def _prunetraceback(self, excinfo):
         Function._prunetraceback(self, excinfo)
         traceback = excinfo.traceback.filter(
-            lambda x: not x.frame.f_globals.get('__unittest'))
+            lambda x: not x.frame.f_globals.get("__unittest")
+        )
         if traceback:
             excinfo.traceback = traceback
 
@@ -198,19 +207,20 @@ def pytest_runtest_makereport(item, call):
             except AttributeError:
                 pass
 
+
 # twisted trial support
 
 
 @hookimpl(hookwrapper=True)
 def pytest_runtest_protocol(item):
-    if isinstance(item, TestCaseFunction) and \
-       'twisted.trial.unittest' in sys.modules:
-        ut = sys.modules['twisted.python.failure']
+    if isinstance(item, TestCaseFunction) and "twisted.trial.unittest" in sys.modules:
+        ut = sys.modules["twisted.python.failure"]
         Failure__init__ = ut.Failure.__init__
         check_testcase_implements_trial_reporter()
 
-        def excstore(self, exc_value=None, exc_type=None, exc_tb=None,
-                     captureVars=None):
+        def excstore(
+            self, exc_value=None, exc_type=None, exc_tb=None, captureVars=None
+        ):
             if exc_value is None:
                 self._rawexcinfo = sys.exc_info()
             else:
@@ -218,8 +228,9 @@ def pytest_runtest_protocol(item):
                     exc_type = type(exc_value)
                 self._rawexcinfo = (exc_type, exc_value, exc_tb)
             try:
-                Failure__init__(self, exc_value, exc_type, exc_tb,
-                                captureVars=captureVars)
+                Failure__init__(
+                    self, exc_value, exc_type, exc_tb, captureVars=captureVars
+                )
             except TypeError:
                 Failure__init__(self, exc_value, exc_type, exc_tb)
 
@@ -235,5 +246,6 @@ def check_testcase_implements_trial_reporter(done=[]):
         return
     from zope.interface import classImplements
     from twisted.trial.itrial import IReporter
+
     classImplements(TestCaseFunction, IReporter)
     done.append(1)
